@@ -1,6 +1,6 @@
 from aristotlelib.api_request import set_api_key
 
-from aristotlelib.project import Project, ProjectStatus
+from aristotlelib.project import Project, ProjectStatus, ProjectInputType
 from aristotlelib.local_file_utils import (
     find_lean_project_root,
     validate_local_file_paths,
