@@ -113,6 +113,16 @@ class AristotleRequestClient:
                 )
             response.raise_for_status()
             return response
+        except httpx.HTTPStatusError as e:
+            # Handle 429 (Too Many Requests) specifically
+            if e.response.status_code == 429:
+                raise AristotleAPIError(
+                    "You currently already have 5 projects in progress. Please wait for a project to complete before starting a new one."
+                ) from e
+            # Re-raise other HTTP errors as generic API errors
+            raise AristotleAPIError(
+                f"API request failed with status {e.response.status_code}: {str(e)}"
+            ) from e
         except httpx.RequestError as e:
             raise AristotleAPIError(f"Request failed: {str(e)}") from e
 
