@@ -3,8 +3,10 @@ Command-line interface for Aristotle SDK.
 """
 
 import argparse
+import asyncio
 import sys
 
+from aristotlelib.tui import run_tui
 from aristotlelib.cli.version import get_version
 from aristotlelib.cli.prove_from_file import ProveFromFileAction
 
@@ -22,6 +24,11 @@ def main():
     parser.add_argument(
         "--version", action="version", version=f"aristotlelib {get_version()}"
     )
+    parser.add_argument(
+        "--api-key",
+        type=str,
+        help="Aristotle API key (can also be set via ARISTOTLE_API_KEY environment variable)",
+    )
     subparsers = parser.add_subparsers(dest="command")
     for action_cls in ALL_ACTIONS:
         action = action_cls(subparsers)
@@ -30,8 +37,7 @@ def main():
     args = parser.parse_args()
 
     if not args.command:
-        parser.print_help()
-        return 1
+        return asyncio.run(run_tui(args.api_key))
 
     return args.func(args)
 
