@@ -8,6 +8,9 @@ logger = logging.getLogger("aristotle")
 
 MAX_FILE_SIZE_BYTES = 100 * 1024 * 1024
 
+# Allowed extensions for context files
+CONTEXT_EXTENSIONS = [".lean", ".md", ".txt", ".tex"]
+
 # These imports are included by default, and don't require importing any new file
 STANDARD_LIBRARIES = [
     "Mathlib",
@@ -77,7 +80,12 @@ def find_lean_project_root(start_path: Path) -> Path:
     )
 
 
-def validate_local_file_path(file_path: Path, project_root: Path | None = None) -> None:
+def validate_local_file_path(
+    file_path: Path,
+    project_root: Path | None = None,
+    require_lean: bool = True,
+    allowed_extensions: list[str] | None = None,
+) -> None:
     """
     Validate that a file path is safe to use.
 
@@ -86,6 +94,9 @@ def validate_local_file_path(file_path: Path, project_root: Path | None = None)
     Args:
         file_path: Path to validate
         project_root: Project root directory to ensure file is within (optional)
+        require_lean: If True, require .lean extension (default: True)
+        allowed_extensions: List of allowed file extensions (e.g., ['.txt', '.md', '.tex']).
+                          Only used when require_lean is False.
 
     Raises:
         ValueError: If path is invalid or outside project root
@@ -107,8 +118,14 @@ def validate_local_file_path(file_path: Path, project_root: Path | None = None)
     if not resolved_path.is_file():
         raise ValueError(f"Path {resolved_path} is not a file")
 
-    if resolved_path.suffix != ".lean":
-        raise ValueError(f"File {resolved_path} is not a Lean file")
+    if require_lean:
+        if resolved_path.suffix != ".lean":
+            raise ValueError(f"File {resolved_path} is not a Lean file")
+    elif allowed_extensions is not None:
+        if resolved_path.suffix not in allowed_extensions:
+            raise ValueError(
+                f"File {resolved_path} must have one of the following extensions: {', '.join(allowed_extensions)}"
+            )
 
 
 def validate_local_file_paths(
@@ -194,22 +211,25 @@ def _clean_dependency_path(relative_path: Path) -> Path:
     package_and_path = package.split("/", 1)
     if len(package_and_path) != 2:
         raise ValueError(f"Invalid dependency path structure: {relative_path}")
-    _package_name, path = package_and_path
+    _, path = package_and_path
     return Path(path)
 
 
 def get_files_for_upload(
-    file_paths: list[Path], project_root: Path | None = None
+    file_paths: list[Path], project_root: Path
 ) -> list[tuple[str, bytes, str]]:
     """
     Get files for upload, reading them safely and returning a list of tuples with the file path, contents, and file type.
+
+    project_root must be provided, because uploaded paths must be relative to it.
+    If not running from a valid lean project, the project_root can be set to the top level directory
+    where they're imported from.
     """
     files: list[tuple[str, bytes, str]] = []
     for file_path in file_paths:
         file_content = read_file_safely(file_path)
         # Use relative path from project root
-        if project_root is not None:
-            file_path = file_path.resolve().relative_to(project_root)
+        file_path = file_path.resolve().relative_to(project_root.resolve())
         files.append(
             (str(_clean_dependency_path(file_path)), file_content, "text/plain")
         )
@@ -233,13 +253,23 @@ def _extract_imports(lean_file_path: Path) -> list[str]:
         with lean_file_path.open("r", encoding="utf-8") as f:
             for line in f:
                 stripped_line = line.strip()
+                if not stripped_line.startswith("import "):
+                    continue
                 import_split = stripped_line.split("import ")
                 if len(import_split) != 2:
                     continue
 
-                import_path = import_split[1].strip()
+                logger.debug(f"Import split: {import_split}")
+                # will likely be a nonempty string in any well-formed import statement
+                # technically imports could be on the next line (or have a comment in the middle),
+                # but we'll ignore that for now as nobody does this
+                import_path_with_commment = import_split[1].strip()
+                import_path = import_path_with_commment.split(" ")[0].strip()
 
-                if any(import_path.startswith(lib) for lib in STANDARD_LIBRARIES):
+                if any(
+                    import_path.startswith(lib + ".") or import_path.strip() == lib
+                    for lib in STANDARD_LIBRARIES
+                ):
                     logger.debug(f"Skipping standard library import: {import_path}")
                     continue
 
@@ -442,3 +472,155 @@ def gather_file_imports(
         visited.remove(original_path)
     return visited
 
+
+def get_default_output_path(input_file_path: Path) -> Path:
+    """
+    Get the default output path for a given input file path.
+    """
+    return input_file_path.with_stem(input_file_path.stem + "_aristotle")
+
+
+def gather_context_files_from_folder(
+    folder_path: Path,
+    allowed_extensions: list[str] | None = None,
+    max_depth: int = 10,
+    max_files: int = 100,
+) -> list[Path]:
+    """
+    Recursively gather all context files from a folder.
+
+    Args:
+        folder_path: Root folder to search
+        allowed_extensions: List of allowed file extensions (default: CONTEXT_EXTENSIONS)
+        max_depth: Maximum recursion depth (default: 10)
+        max_files: Maximum number of files to gather (default: 100)
+
+    Returns:
+        List of file paths with allowed extensions, sorted alphabetically
+
+    Raises:
+        ValueError: If folder doesn't exist or is not a directory
+        LeanProjectError: If no valid files found or too many files found
+    """
+    if allowed_extensions is None:
+        allowed_extensions = CONTEXT_EXTENSIONS
+
+    if not folder_path.exists():
+        raise ValueError(f"Folder does not exist: {folder_path}")
+
+    if not folder_path.is_dir():
+        raise ValueError(f"Path is not a directory: {folder_path}")
+
+    gathered_files: list[Path] = []
+    visited_dirs: set[Path] = set()  # Track visited to avoid symlink cycles
+
+    _gather_files_recursive(
+        folder_path,
+        gathered_files,
+        visited_dirs,
+        allowed_extensions,
+        current_depth=0,
+        max_depth=max_depth,
+        max_files=max_files,
+    )
+
+    if not gathered_files:
+        ext_str = ", ".join(allowed_extensions)
+        raise LeanProjectError(
+            f"No files with extensions ({ext_str}) found in folder: {folder_path}"
+        )
+
+    # Sort alphabetically for consistent ordering
+    gathered_files.sort()
+
+    logger.info(f"Gathered {len(gathered_files)} context files from {folder_path}")
+    return gathered_files
+
+
+def _gather_files_recursive(
+    current_path: Path,
+    gathered_files: list[Path],
+    visited_dirs: set[Path],
+    allowed_extensions: list[str],
+    current_depth: int,
+    max_depth: int,
+    max_files: int,
+) -> None:
+    """
+    Recursive helper for gather_context_files_from_folder.
+
+    Args:
+        current_path: Current directory being processed
+        gathered_files: Accumulated list of files (modified in place)
+        visited_dirs: Set of visited directories (for cycle detection)
+        allowed_extensions: List of allowed file extensions
+        current_depth: Current recursion depth
+        max_depth: Maximum allowed depth
+        max_files: Maximum allowed files
+
+    Raises:
+        LeanProjectError: If too many files found
+    """
+    # Check depth limit
+    if current_depth > max_depth:
+        logger.warning(f"Max depth {max_depth} reached at {current_path}, skipping")
+        return
+
+    # Resolve path and check for cycles
+    try:
+        resolved_path = current_path.resolve()
+    except (OSError, RuntimeError):
+        logger.warning(f"Cannot resolve path {current_path}, skipping")
+        return
+
+    if resolved_path in visited_dirs:
+        logger.debug(f"Already visited {resolved_path}, skipping (symlink cycle?)")
+        return
+
+    visited_dirs.add(resolved_path)
+
+    # List directory contents
+    try:
+        items = sorted(current_path.iterdir())
+    except PermissionError:
+        logger.warning(f"Permission denied reading {current_path}, skipping")
+        return
+    except Exception as e:
+        logger.warning(f"Error reading {current_path}: {e}, skipping")
+        return
+
+    for item in items:
+        # Skip hidden files/folders
+        if item.name.startswith('.'):
+            continue
+
+        # Skip common non-relevant directories
+        if item.is_dir() and item.name in {
+            'node_modules', '__pycache__', '.git', '.lake',
+            'build', 'dist', 'target', '.venv', 'venv',
+        }:
+            continue
+
+        if item.is_file():
+            # Check if file has allowed extension
+            if item.suffix.lower() in allowed_extensions:
+                gathered_files.append(item.resolve())
+
+                # Check file count limit
+                if len(gathered_files) > max_files:
+                    raise LeanProjectError(
+                        f"Too many files found (>{max_files}). "
+                        f"Please specify a more specific folder or reduce the scope."
+                    )
+
+        elif item.is_dir():
+            # Recurse into subdirectory
+            _gather_files_recursive(
+                item,
+                gathered_files,
+                visited_dirs,
+                allowed_extensions,
+                current_depth + 1,
+                max_depth,
+                max_files,
+            )
